#' @docType data
#' @title Sample dataset from the National Health and Nutrition Examination Survey
#' @name nhanes_sample
#' @keywords data
#'
#' @description Selected variables from the National Health and Nutrition Examination
#'              Survey that are used in the example from Lumley (2010), Appendix E.
#'              See \code{\link{svyglm.nb}} for examples.
#'
#' @references Lumley T (2010). Complex Surveys: a guide to analysis using R. Wiley
NULL

#' @docType data
#' @title Sample dataset from the EUROFAMCARE project
#' @name efc
#' @keywords data
#'
#' @description German data set from the European study on family care of older people.
#'
#' @references Lamura G, Döhner H, Kofahl C, editors. Family carers of older people in Europe: a six-country comparative study. Münster: LIT, 2008.
NULL
