% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nhanes_sample.R
\docType{data}
\name{efc}
\alias{efc}
\title{Sample dataset from the EUROFAMCARE project}
\description{
German data set from the European study on family care of older people.
}
\references{
Lamura G, Döhner H, Kofahl C, editors. Family carers of older people in Europe: a six-country comparative study. Münster: LIT, 2008.
}
\keyword{data}
